
import time
import sys
from pawt import swing
from java.awt import GridBagLayout, GridBagConstraints, FlowLayout
from pawt import GridBag
APPROVE_OPTION = swing.JFileChooser.APPROVE_OPTION
import java.io
import pickle
import os
from Options import OptionsTable
import coopr.pyomo.scripting
from threading import Thread
import pyutilib.misc
import pyutilib.subprocess
import StringIO
from javax.swing import SwingWorker, SwingUtilities
from java.util.concurrent import CancellationException
from java.util.concurrent import ExecutionException
from java.lang import Runtime, Runnable, ProcessBuilder
from java.io import InputStreamReader, BufferedReader
from java.awt import Font



class Container(dict):
    """
    A generalization of Bunch.  This class allows all other attributes to have a
    default value of None.  This borrows the output formatting ideas from the 
    ActiveState Code Container (recipe 496697).
    """

    def __init__(self, *args, **kw):
        for arg in args:
            for item in quote_split('[ \t]+',arg):
                r = item.find('=')
                if r != -1:
                    try:
                        val = eval(item[r+1:])
                    except:
                        val = item[r+1:]
                    kw[item[:r]] = val
        dict.__init__(self,kw)
        self.__dict__.update(kw)
        if not '_name_' in kw:
            self._name_ = self.__class__.__name__

    def set_name(self, name):
        self._name_ = name

    def __setitem__(self, name, val):
        self.__setattr__(name,val)

    def __getitem__(self, name):
        return self.__getattr__(name)

    def __setattr__(self, name, val):
        if name[0] != '_':
            dict.__setitem__(self, name, val)
        self.__dict__[name] = val

    def __getattr__(self, name):
        try:
            return dict.__getitem__(self, name)
        except:
            if name[0] == '_':
                raise AttributeError, "Unknown attribute %s" % name
        return None

    def __repr__(self):
        attrs = sorted("%s = %r" % (k, v) for k, v in self.__dict__.iteritems() if not k.startswith("_"))
        return "%s(%s)" % (self.__class__.__name__, ", ".join(attrs))

    def __str__(self):
        return self.as_string()

    def __str__(self, nesting = 1, indent='', print_name=True):
        attrs = []
        if not print_name:
            nesting -= 1
        indentation = indent+"    " * nesting
        for k, v in self.__dict__.iteritems():
            if not k.startswith("_"):
                text = [indentation, k, " = "]
                if isinstance(v, Container):
                    text.append(v.__str__(nesting + 1))
                else:
                    text.append(repr(v))
                attrs.append("".join(text))
        attrs.sort()
        if print_name:
            attrs.insert(0, indent+self._name_ + ":")
        return "\n".join(attrs)


class PyomoWorker(SwingWorker):
    "Class implementing long running task as a SwingWorker thread"

    def __init__(self, options, args, parser, gui, shell=True):
        SwingWorker.__init__(self)
        self.options=options
        self.args=args
        self.parser=parser
        self.shell=shell
        self.gui=gui
        self.cmd=[]
        self.stdout_reader_done = False
        self.stderr_reader_done = False

    def doInBackground(self):
	pyomocmd = os.path.dirname(sys.executable)+os.sep+"pyomo"
        self.cmd = [sys.executable, pyomocmd]
        for opt in self.parser.option_list:
            if len(opt._long_opts) > 0:
                key = opt._long_opts[0][2:]
            else:
                key = opt._short_opts[0][1:]
            if self.options[key] is None:
                continue
            if self.options[key] == opt.default:
                continue
            if type(self.options[key]) is list:
                for item in self.options[key]:
                    self.cmd.append("--"+key)
		    self.cmd.append(str(item))
            else:
                self.cmd.append("--"+key)
		self.cmd.append(str(self.options[key]))
	for arg in self.args[1:]:
	    if arg != '':
                self.cmd.append(arg)
        self.update("Executing the following command:\n\n  "+' '.join(self.cmd)+"\n", self.gui)

	def stdout_reader(stream, self):
	    s = ''
	    while (s != None):
		if self.isCancelled():
		    self.stdout_reader_done=True
	            return
                s = stream.readLine()
		if s is None:
		    self.stdout_reader_done=True
		    return
	        else:
	            self.update(s+"\n", self.gui)
	    self.stdout_reader_done=True

	def stderr_reader(stream, buffer, self):
	    s = ''
	    while (s != None):
		if self.isCancelled():
		    self.stderr_reader_done=True
	            return
                s = stream.readLine()
		if s is None:
		    self.stderr_reader_done=True
		    return
	        else:
	            buffer += s
	    self.stderr_reader_done=True

	self.cmd = ['cmd.exe', '/C', 'call'] + self.cmd

	builder = ProcessBuilder(self.cmd)
	proc = builder.start()
        stdInput = BufferedReader(InputStreamReader(proc.getInputStream()))
        stdError = BufferedReader(InputStreamReader(proc.getErrorStream()))
	#
	try:
            self.update("\n--- PYOMO START STDOUT ---\n\n", self.gui)
	    self.buffer=''
	    self.outt = Thread(target=stdout_reader, args=(stdInput,self))
	    self.outt.daemon = True
	    self.outt.start()
	    self.errt = Thread(target=stderr_reader, args=(stdError,self.buffer,self))
	    self.errt.daemon = True
	    self.errt.start()
	    ##print "HERE 2"
	    self.outt.join()
	    ##print "HERE 3"
	    self.errt.join()
	    ##print "HERE 4"
	    self.outt = None
	    self.errt = None
        except Exception, e:
	    raise e
	    ##print "HERE 5",str(e)
	finally:
	    ##print "HERE 6"
	    self.final_output()
	    self.finish(self.gui)
    
    def final_output(self):
    	#while not self.stdout_reader_done or not self.stderr_reader_done:
	    #print "HERE Y"
	    #time.sleep(0.1)
	print "HERE X"
	if self.isCancelled():
	    self.update("--- Pyomo Solver Terminated Early!  ----\n", self.gui)
        self.update("\n--- PYOMO END STDOUT ---\n", self.gui)
	#
	if len(self.buffer) > 0:
            self.update("\n\n--- PYOMO START STDERR ---\n\n", self.gui)
	    self.update(self.buffer, self.gui)
            self.update("\n--- PYOMO END STDERR ---\n", self.gui)

    def finish(self, gui):
        class updater( java.lang.Runnable ) :
            def run( self ) :
        	gui._setButtonStates(started=False, finished=True)
        if SwingUtilities.isEventDispatchThread() :
            updater().run()
        else :
            SwingUtilities.invokeLater( updater() )

    def update(self, text, gui):
	#print "X",text
	tab = gui.tab_info[gui.tabbed.getSelectedIndex()]
        # now update the GUI,  create an java.lang.Runnable that
        # does the work
        class updater( java.lang.Runnable ) :
            def run( self ) :
                tab.output.append(text)
	#
        if SwingUtilities.isEventDispatchThread() :
            # we are already in the AWT_EventQueue thread, it is
            # safe to touch the GUI
            updater().run()
        else :
            # request that the updater be run in the
            # AWT_EventQueue thread
            SwingUtilities.invokeLater( updater() )
	###print "UPDATE - done"

    def process(self, text):
        self.gui.output.append(text)

    def done(self):
        try:
            self.get()
        except CancellationException, e:
	    # ignore exception caused when worker is explicitly canceled
	    pass
        except ExecutionException, e:
            raise SystemExit, e.getCause()



class GUI:

    def __init__(self):
        self.tab_id=0
        self.tab_info = {}
        self.frame = swing.JFrame("Pyomo Command Driver", windowClosing=self._fileQuit)
        self.frame.JMenuBar = self.create_menubar()

        self.bag = GridBag(self.frame.contentPane, \
                        fill='BOTH', weightx=1.0)

        # do the rest of the layout in a GridBag
        #self.do_layout(self.bag)
        self.tabbed = swing.JTabbedPane()
        self.create_tab()
        self.bag.addRow(self.tabbed, weighty=1.0)
        self.bag.addRow(swing.JLabel("Pyomo version VERSION (DATE)"))

        # Create a file chooser
        self.chooser = swing.JFileChooser()
        self.chooser.currentDirectory = java.io.File(os.getcwd())

        # Finish up and set the size
        self.frame.pack()
	# ACTIVATE AND THEN SET VISIBLE
        self.frame.setVisible(1)
        self.frame.size = self.frame.getPreferredSize()

    def create_tab(self):
        self.tab_id += 1
        tab = Container()
        self.tab_info[self.tab_id-1] = tab
        #
        # Setting up the panel for the tab
        #
        panel = swing.JPanel()
        bag = GridBag(panel, fill='BOTH', weightx=1.0)
        self.tabbed.add(panel, 'Tab %d' % self.tab_id)
        #
        # Creating button for model
        #
        input1 = swing.JPanel(FlowLayout(alignment=FlowLayout.LEFT))
        tab.buttonInput1 = swing.JButton("Pyomo Model File", actionPerformed=self._buttonInput1)
        input1.add(tab.buttonInput1)
        bag.addRow(input1)
        tab.text1 = swing.JTextField("",60)
        bag.addRow(tab.text1)
        #
        # Create button for data
        #
        input2 = swing.JPanel(FlowLayout(alignment=FlowLayout.LEFT))
        tab.buttonInput2 = swing.JButton("Pyomo Data File", actionPerformed=self._buttonInput2)
        input2.add(tab.buttonInput2)
        bag.addRow(input2)
        tab.text2 = swing.JTextField("",60)
        bag.addRow(tab.text2)
        #
        # create panel for the options in the command line
        #
        options_panel = swing.JPanel(FlowLayout(alignment=FlowLayout.LEFT))
        tab.parser = coopr.pyomo.scripting.pyomo.create_parser()
        tab.parse_options = tab.parser.parse_args(args=['pyomo'])[0]
        tab.options_pane, tab.options_table = self.create_options_table(tab.parser, tab.parse_options)
        options_panel.add(tab.options_pane)
        bag.addRow(options_panel)
        #
        # create panel for buttons that manage command execution/output
        #
        buttons = swing.JPanel(FlowLayout(alignment=FlowLayout.LEFT))
        tab.buttonExecute = swing.JButton("Execute", actionPerformed=self._buttonExecute)
        buttons.add(tab.buttonExecute)
        tab.buttonStop = swing.JButton("Stop", actionPerformed=self._buttonStop)
        buttons.add(tab.buttonStop)
        tab.buttonClearOutput = swing.JButton("Clear Output", actionPerformed=self._buttonClearOutput)
        buttons.add(tab.buttonClearOutput)
        bag.addRow(buttons)
        self._setButtonStates(started=False, finished=True)
        #
        tab.output = swing.JTextArea("",15,80)
	tab.output.setFont( Font('Courier New', Font.PLAIN, 12) )
	#tab.output.setFont( Font('Lucida Console', Font.PLAIN, 12) )
        pane = swing.JScrollPane(tab.output,\
                    verticalScrollBarPolicy=swing.JScrollPane.VERTICAL_SCROLLBAR_ALWAYS)
                    #size=(1000,10000))
        bag.addRow(pane, weighty=1.0)
        #tab.output.append("DRAFT GUI for PYOMO COMMAND DRIVER\n\n")

    def create_options_table(self, parser, parse_options):
        options_table = OptionsTable()
        options_table.initialize(parser, parse_options)
        scrollPane = swing.JScrollPane()
        scrollPane.setPreferredSize(java.awt.Dimension(600,100))
        scrollPane.setMinimumSize(java.awt.Dimension(300,100))
        scrollPane.getViewport().setView(options_table)
        return scrollPane, options_table

    def create_menubar(self):
        """build menu bar"""
        menubar = swing.JMenuBar()
        #
        file = swing.JMenu("File")
        file.add(swing.JMenuItem("New Command", actionPerformed = self._fileNewCommand))
        file.add(swing.JMenuItem("Load...", actionPerformed = self._fileLoad))
        file.add(swing.JMenuItem("Save...", actionPerformed = self._fileSave))
        file.add(swing.JMenuItem("Save to file...", actionPerformed = self._fileSaveToFile))
        file.add(swing.JMenuItem("Quit", actionPerformed = self._fileQuit))
        menubar.add(file)
        #
        file = swing.JMenu("Edit")
        file.add(swing.JMenuItem("Copy", actionPerformed = self._editCopy))
        file.add(swing.JMenuItem("Paste", actionPerformed = self._editPaste))
        file.add(swing.JMenuItem("Quit", actionPerformed = self._editSelectAll))
        menubar.add(file)
        #
        ##file = swing.JMenu("Packages")
        ##file.add(swing.JMenuItem("Versions", actionPerformed = self._packagesVersions))
        ##file.add(swing.JMenuItem("Install...", actionPerformed = self._packagesInstall))
        ##file.add(swing.JMenuItem("Update...", actionPerformed = self._packagesUpdate))
        ##menubar.add(file)
        #
        file = swing.JMenu("Help")
        file.add(swing.JMenuItem("Console", actionPerformed = self._helpConsole))
        #file.add(swing.JMenuItem("COIN-OR wiki", actionPerformed = self._helpCoinWiki))
        #file.add(swing.JMenuItem("Coopr wiki", actionPerformed = self._helpCooprWiki))
        file.add(swing.JMenuItem("About", actionPerformed = self._helpAbout))
        menubar.add(file)
        return menubar

    def _setButtonStates(self, started, finished):
        tab = self.tab_info[self.tabbed.getSelectedIndex()]
        tab.buttonExecute.enabled = not started
        tab.buttonStop.enabled = started
        tab.buttonClearOutput.enabled = finished

    def _fileNewCommand(self, event=None):
        self.create_tab()

    def _fileLoad(self, event=None):
        file,absfile = self._select_file()
        if not absfile is None:
            INPUT = open(absfile)
            (a,b) = pickle.load(INPUT)
            INPUT.close()

    def _fileSave(self, event=None):
        file,absfile = self._select_file()
        if not absfile is None:
            object = ("AA",  "BB")
            OUTPUT = open(absfile, 'w')
            pickle.dump(object, OUTPUT)
            OUTPUT.close()

    def _fileSaveToFile(self, event=None):
        pass

    def _fileQuit(self, event=None):
        import sys
        sys.exit(0)

    def _editCopy(self, event=None):
        pass

    def _editPaste(self, event=None):
        pass

    def _editSelectAll(self, event=None):
        pass

    def _helpConsole(self, event=None):
        pass

    def _helpCoinWiki(self, event=None):
        pass

    def _helpCooprWiki(self, event=None):
        pass

    def _helpAbout(self, event=None):
        pass

    def _packagesVersions(self, event=None):
        pass

    def _packagesInstall(self, event=None):
        pass

    def _packagesUpdate(self, event=None):
        pass

    def _buttonExecute(self, event=None):
        self._setButtonStates(started=True, finished=False)
        tab = self.tab_info[self.tabbed.getSelectedIndex()]
        tab.output.text = ""
        options = tab.options_table.get_options()
        tab.PyomoWorker = PyomoWorker(options, ['pyomo', tab.text1.text, tab.text2.text], tab.parser, self)
        tab.PyomoWorker.execute()

    def _buttonStop(self, event=None):
        #self._setButtonStates(started=False, finished=True)
        tab = self.tab_info[self.tabbed.getSelectedIndex()]
        tab.PyomoWorker.cancel(True)
        #tab.PyomoWorker = None

    def _buttonClearOutput(self, event=None):
        self.tab_info[self.tabbed.getSelectedIndex()].output.text = ""

    def _select_file(self):
        self.chooser.rescanCurrentDirectory()
        returnVal = self.chooser.showOpenDialog(self.frame)
        if returnVal == APPROVE_OPTION:
            return (self.chooser.selectedFile.name, os.path.join(self.chooser.currentDirectory.path, self.chooser.selectedFile.name))
        return (None,None)
        
    def _buttonInput1(self, event=None):
        file, absfile = self._select_file()
        self.tab_info[self.tabbed.getSelectedIndex()].text1.text = absfile

    def _buttonInput2(self, event=None):
        file, absfile = self._select_file()
        self.tab_info[self.tabbed.getSelectedIndex()].text2.text = absfile


def run():
    GUI()

if __name__ == '__main__':
    run()
