__all__ = ['OptionsTable']

import javax.swing as swing
import string
from pyutilib.misc import Options
import re


class StringOption:

    def __init__(self, name, default):
        self.name = name
        self.default = default
        self.editor = swing.JTextField("", 250)

    def validate(self, value):
	return isinstance(value,basestring)


class IntegerOption:

    def __init__(self, name, default):
        self.name = name
	if isinstance(default,basestring):
	    self.default = string.atoi(default)
	else:
	    self.default = default

        self.editor = swing.JTextField("", 50)

    def validate(self, value):
	try:
	    val = int(value)
	    return True
	except:
	    return False


class LongOption:

    def __init__(self, name, default):
        self.name = name
	if isinstance(default,basestring):
	    self.default = string.atol(default)
	else:
	    self.default = default
        self.editor = swing.JTextField("", 50)

    def validate(self, value):
	try:
	    val = long(value)
	    return True
	except:
	    return False


class FloatOption:

    def __init__(self, name, default):
        self.name = name
	if isinstance(default,basestring):
	    self.default = string.atof(default)
	else:
	    self.default = default
        self.editor = swing.JTextField("", 50)

    def validate(self, value):
	try:
	    val = float(value)
	    return True
	except:
	    return False


class EnumOption:

    def __init__(self, name, default, list_of_values):
        self.name = name
	self.default = default
        self.values = list_of_values
        self.editor = swing.JComboBox(self.values)

    def validate(self, value):
	return value in self.values


class OptionsTable(swing.JTable):

    def __init__(self):
        self.columns = ('Option', 'Value')
        self.data = list()
        self.options = list()

    def initialize(self, parser, options):
	self.parser = parser
	#print "HERE",parser,dir(parser)
	#print "HERE",options,dir(options)
	for opt in parser.option_list:
	    #print dir(opt)
	    if len(opt._long_opts) > 0:
		key = opt._long_opts[0][2:]
	    else:
		key = opt._short_opts[0][1:]
	    #print "X",key,opt.default,opt.type
	    if opt.type == 'string':
		if opt.default in [False,True]:
		    if opt.default:
			default='True'
		    else:
			default='False'
		    self.options.append( EnumOption(key, default, ['True','False']))
		    self.data.append((key,default))
		else:
		    if type(opt.default) is list:
			default=""
		    else:
			default=opt.default
		    self.options.append( StringOption(key, default) )
		    self.data.append((key, default))
	    elif opt.type == 'int':
		self.options.append( IntegerOption(key, opt.default) )
	    elif opt.type == 'float':
		self.options.append( FloatOption(key, opt.default) )
		self.data.append((key,opt.default))
	    elif opt.type == 'long':
		self.options.append( LongOption(key, opt.default) )
		self.data.append((key,opt.default))
	    elif opt.type == 'choice':
		if type(opt.default) is list:
		    default=""
		else:
		    default=opt.default
		self.options.append( EnumOption(key, default, opt.choices) )
		self.data.append((key, default))
	#
        self.setShowGrid(True)
        self.setShowVerticalLines(True)
        self.getModel().setDataVector(self.data, self.columns)

    def get_options(self):
	options = Options()
	i=0
	for data in self.data:
	    opt = self.parser.get_option('--'+data[0])
	    name = data[0]
	    #print "X", name, self.getValueAt(i,0), self.getValueAt(i,1)
	    rawvalue = self.getValueAt(i,1)
	    if isinstance(rawvalue,basestring):
	        rawvalue = rawvalue.strip()
	    #print "Y",dir(opt)
	    if opt.action == 'append':
		options[name] = re.split('[ \t]+', rawvalue)
		if options[name] == ['']:
		    options[name] = []
		elif opt.type == 'int':
		    options[name] == [int(val) for val in options[name]]
		elif opt.type == 'long':
		    options[name] == [long(val) for val in options[name]]
		elif opt.type == 'float':
		    options[name] == [float(val) for val in options[name]]
	    else:
		options[name] = rawvalue
		if opt.type == 'int':
		    if options[name] == '':
			options[name] == None
		    elif not options[name] is None:
			options[name] == int(options[name])
		elif opt.type == 'long':
		    if options[name] == '':
			options[name] == None
		    elif not options[name] is None:
			options[name] == long(options[name])
		elif opt.type == 'float':
		    if options[name] == '':
			options[name] == None
		    elif not options[name] is None:
			options[name] == float(options[name])
	    i += 1
	return options

    def Xread_options_list(self, fname):
        try:
            f = open(fname, 'r')
        except IOError:
            raise NameError, "Error opening dialog file"

        line = f.readline()
        while line != "":
            line.lstrip()
            line.rstrip()
            if line.startswith('#'):
                line = f.readline()
                continue

            tokens = line.split(',')
            name = tokens[0]
            type = tokens[1]
            if type == 'integer':
                self.options.append(IntegerOption(name,tokens[2],tokens[3],tokens[4]))
                self.data.append((name, tokens[2], tokens[3], tokens[4], ""))
            elif type == 'float':
                self.options.append(FloatOption(name,tokens[2],tokens[3],tokens[4]))
                self.data.append((name, tokens[2], tokens[3], tokens[4], ""))
            elif type == 'enum':
                default = tokens[2]
                possible_values = list()
                for i in range(3,len(tokens)):
                    possible_values.append(tokens[i])
                    
                self.options.append(EnumOption(name,possible_values))
                self.data.append((name, '-', str(default), '-', ""))
            else:
                raise NameError, "Invalid type found in dialog file"
            line = f.readline()
            
        f.close()
        self.setShowGrid(True)
        self.setShowVerticalLines(True)
        self.getModel().setDataVector(self.data, self.columns)

    def getCellEditor(self, row=-1, col=-1):
	if row == -1:
        	return swing.JTable.getCellEditor(self)
	if col==0:
	    return None
        if row < len(self.options):
            if isinstance(self.options[row], EnumOption):
                return swing.DefaultCellEditor(self.options[row].editor)
        return swing.JTable.getCellEditor(self, row, col)

